#!/bin/bash
set -euo pipefail  # 严格模式：错误退出、未定义变量报错、管道错误检测

# ========== CICD主脚本 ==========
# 该脚本负责读取配置、更新代码并调用其他功能脚本

# 脚本所在目录
SCRIPT_DIR="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)"

# ========== 参数解析 ==========
# 必要参数
if [ $# -lt 3 ]; then
    echo "用法: $0 <环境> <项目名> <子工程名> [分支名]"
    echo "示例: $0 test weef-iot-edgexapp weef-starter-edge-biz develop"
    exit 1
fi

ENV="$1"          # 环境：test, prod
PROJECT="$2"      # 项目名（gitlab库名）
SUB_PROJECT="$3"  # 子工程名
CMD_BRANCH="${4:-}"   # 命令行传入的分支名（可选）
# 优先使用环境变量中的BRANCH，其次是命令行参数
BRANCH="${BRANCH:-$CMD_BRANCH}"   # 实际使用的分支名

# ========== 加载配置文件 ==========
# 加载环境通用配置
ENV_CONFIG="$SCRIPT_DIR/config/$ENV/$ENV.env"
if [ ! -f "$ENV_CONFIG" ]; then
    echo "错误：环境配置文件不存在: $ENV_CONFIG"
    exit 1
fi
source "$ENV_CONFIG"

# 加载项目子工程配置
PROJECT_CONFIG="$SCRIPT_DIR/config/$ENV/$PROJECT/$SUB_PROJECT.env"
if [ ! -f "$PROJECT_CONFIG" ]; then
    echo "错误：项目配置文件不存在: $PROJECT_CONFIG"
    exit 1
fi
source "$PROJECT_CONFIG"

# 设置分支（环境变量优先，其次是命令行参数，然后是配置文件，最后是默认值）
GIT_BRANCH="${BRANCH:-${GIT_BRANCH:-$GIT_DEFAULT_BRANCH}}"

# ========== 设置衔生变量 ==========
# 代码目录结构
PROJECT_DIR="$CODE_BASE_DIR/$PROJECT"
CODE_DIR="$PROJECT_DIR/$GIT_BRANCH"

# 确保目录存在
mkdir -p "$CODE_DIR"

# JAR包相关路径
JAR_NAME="$PROJECT_SUB_NAME.jar"
# LOCAL_JAR变量初始设置，可能会在java-compile.sh中被更新
LOCAL_JAR="$CODE_DIR/$PROJECT/$PROJECT_SUB_NAME/target/$JAR_NAME"
REMOTE_TMP="/root/.update/$JAR_NAME"
CONTAINER_APP_PATH="/$JAR_NAME"

# 镜像名称
SOURCE_IMAGE="com.weef.nacos/$PROJECT_SUB_NAME"
TARGET_IMAGE="192.168.3.48:5000/inxvision/$PROJECT_SUB_NAME"
IMAGE_VERSION="${IMAGE_VERSION:-$DEFAULT_IMAGE_VERSION}"
IMAGE_PATTERN="inxvision/$PROJECT_SUB_NAME"

# 功能开关（命令行环境变量优先，其次是配置文件，最后是默认值）
UPDATE_IMAGE="${UPDATE_IMAGE:-$DEFAULT_UPDATE_IMAGE}"
UPDATE_REMOTE_SERVICE="${UPDATE_REMOTE_SERVICE:-$DEFAULT_UPDATE_REMOTE_SERVICE}"

# 远程主机
REMOTE_HOST="${REMOTE_HOST:-$DEFAULT_REMOTE_HOST}"

# ========== 导出所有变量供子脚本使用 ==========
export JAVA_HOME
export GIT_REPO
export GIT_BRANCH
export CODE_BASE_DIR
export PROJECT_DIR
export BRANCH_DIR
export CODE_DIR
export PROJECT_SUB_NAME
export PROJECT  # 导出项目名变量
export SUB_PROJECT  # 导出子工程名变量
export ENV  # 导出环境变量
export JAR_NAME
export LOCAL_JAR
export REMOTE_TMP
export CONTAINER_APP_PATH
export DOCKER_REGISTRY
export DOCKER_USERNAME
export DOCKER_PASSWORD
export SOURCE_IMAGE
export TARGET_IMAGE
export IMAGE_VERSION
export IMAGE_PATTERN
export IS_PROD
export REMOTE_HOST
export RESTART_WAIT_TIME
export UPDATE_IMAGE
export UPDATE_REMOTE_SERVICE

# ========== 显示配置信息 ==========
echo "====== CICD配置信息 ======"
echo "环境: $ENV"
echo "项目: $PROJECT"
echo "子工程: $PROJECT_SUB_NAME"
echo "分支: $GIT_BRANCH (来源: ${BRANCH+环境变量}${BRANCH-${CMD_BRANCH+命令行参数}${CMD_BRANCH-配置文件}})"
echo "原始分支参数: BRANCH=${BRANCH:-未设置}, CMD_BRANCH=${CMD_BRANCH:-未设置}, GIT_BRANCH(配置文件)=${GIT_BRANCH:-未设置}"
echo "代码目录: $CODE_DIR"
echo "更新镜像: $UPDATE_IMAGE"
echo "更新远程服务: $UPDATE_REMOTE_SERVICE"
echo "=========================="

# ========== 主流程 ==========
main() {
    echo "====== 开始执行 $PROJECT_SUB_NAME 部署流程 [$(date +'%Y-%m-%d %H:%M:%S')] ======"

    # 检查功能脚本是否存在
    JAVA_COMPILE_SCRIPT="$SCRIPT_DIR/java-compile.sh"
    PUSH_IMAGE_SCRIPT="$SCRIPT_DIR/push-image.sh"
    RESTART_SERVICE_SCRIPT="$SCRIPT_DIR/restart-service.sh"

    for script in "$JAVA_COMPILE_SCRIPT" "$PUSH_IMAGE_SCRIPT" "$RESTART_SERVICE_SCRIPT"; do
        if [ ! -f "$script" ]; then
            echo "错误：功能脚本不存在: $script"
            exit 1
        fi
        # 确保脚本有执行权限
        chmod +x "$script"
    done

    # 执行Java编译
    echo "→ 调用Java编译脚本..."
    "$JAVA_COMPILE_SCRIPT"

    # 根据配置决定是否更新镜像
    if [ "$UPDATE_IMAGE" = "true" ]; then
        echo "→ 调用Docker镜像推送脚本..."
        "$PUSH_IMAGE_SCRIPT"
    else
        echo "⹷ 跳过镜像更新（UPDATE_IMAGE=false）"
    fi

    # 根据配置决定是否更新远程服务
    if [ "$UPDATE_REMOTE_SERVICE" = "true" ]; then
        echo "→ 调用服务重启脚本..."
        "$RESTART_SERVICE_SCRIPT"
    else
        echo "⹷ 跳过远程服务更新（UPDATE_REMOTE_SERVICE=false）"
    fi

    echo "====== $PROJECT_SUB_NAME 部署流程执行完毕 [总耗时: $SECONDS 秒] ======"
}

# 执行主函数
main