#!/bin/bash
set -euo pipefail  # 严格模式：错误退出、未定义变量报错、管道错误检测

# ========== Java编译脚本 ==========
# 该脚本负责更新代码并编译Java项目

# 检查必要的环境变量
check_required_vars() {
    local required_vars=("CODE_DIR" "GIT_REPO" "GIT_BRANCH" "PROJECT_SUB_NAME" "JAVA_HOME")
    for var in "${required_vars[@]}"; do
        if [ -z "${!var:-}" ]; then
            echo "错误：缺少必要的环境变量 $var"
            exit 1
        fi
    done
    echo "✓ 环境变量检查通过"
}

# 更新代码
update_code() {
    echo "→ 开始处理代码仓库..."
    cd "$CODE_BASE_DIR" || { echo "错误：目录 $CODE_BASE_DIR 不存在"; exit 1; }

    # 如果存在任何旧目录，先清理
    if [ -d "$PROJECT_DIR" ]; then
        echo "清理现有项目目录..."
        rm -rf "$PROJECT_DIR"
    fi

    # 创建项目目录和分支目录
    mkdir -p "$CODE_DIR"

    echo "下载代码库到 $CODE_DIR..."

    # 直接克隆到目标目录
    git clone -b "$GIT_BRANCH" "$GIT_REPO" "$CODE_DIR"

    # 检查克隆后的目录结构
    echo "→ 克隆后的目录结构:"
    ls -la "$CODE_DIR"

    echo "✓ 代码更新完成"
}

# 编译项目
compile_project() {
    echo "→ 开始编译项目..."
    cd "$CODE_DIR/$PROJECT" || { echo "错误：代码目录不存在"; exit 1; }

    # 列出目录结构，帮助调试
    echo "→ 当前目录结构:"
    ls -la

    # 显示环境变量
    echo "→ JAVA_HOME: $JAVA_HOME"

    # 显示Java版本
    echo "→ Java版本信息:"
    java -version || echo "警告：无法显示Java版本信息"

    # 执行Maven编译
    echo "→ 执行Maven编译..."
    mvn clean install -Pjar -Dmaven.test.skip=true

    # 检查编译结果
    if [ ! -f "$PROJECT_SUB_NAME/target/$JAR_NAME" ]; then
        echo "警告：预期的JAR包不存在: $PROJECT_SUB_NAME/target/$JAR_NAME"
        echo "尝试查找其他可能的位置..."

        # 尝试查找JAR包
        JAR_PATH=$(find "$CODE_DIR/$PROJECT/$PROJECT_SUB_NAME" -name "$JAR_NAME" | grep "/target/" | head -1)

        if [ -n "$JAR_PATH" ]; then
            echo "✓ 找到JAR包: $JAR_PATH"
            # 更新LOCAL_JAR变量
            export LOCAL_JAR="$JAR_PATH"
        else
            echo "错误：无法找到编译后的JAR包: $JAR_NAME"
            exit 1
        fi
    else
        # 更新LOCAL_JAR变量
        export LOCAL_JAR="$(pwd)/$PROJECT_SUB_NAME/target/$JAR_NAME"
        echo "✓ JAR包路径: $LOCAL_JAR"
    fi

    echo "✓ 编译完成"
}

# ========== 主流程 ==========
main() {
    echo "====== 开始执行Java编译流程 [$(date +'%Y-%m-%d %H:%M:%S')] ======"
    check_required_vars
    update_code
    compile_project
    echo "====== Java编译流程执行完毕 [耗时: $SECONDS 秒] ======"
}

# 执行主函数
main