#!/bin/bash
set -euo pipefail  # 严格模式：错误退出、未定义变量报错、管道错误检测

# ========== Docker镜像推送脚本 ==========
# 该脚本负责构建和推送Docker镜像

# 检查必要的环境变量
check_required_vars() {
    # 检查基本变量
    local required_vars=("SOURCE_IMAGE" "TARGET_IMAGE" "IMAGE_VERSION" "PROJECT_SUB_NAME")

    # 如果是生产环境，则需要登录凭证
    if [ "${IS_PROD:-true}" = "true" ]; then
        required_vars+=("DOCKER_REGISTRY" "DOCKER_USERNAME" "DOCKER_PASSWORD")
    fi

    for var in "${required_vars[@]}"; do
        if [ -z "${!var:-}" ]; then
            echo "错误：缺少必要的环境变量 $var"
            exit 1
        fi
    done
    echo "✓ 环境变量检查通过"
}
make_image(){
    echo "→ 开始构建镜像"
    cd $CODE_DIR/$PROJECT/$PROJECT_SUB_NAME
    mvn clean install -Dmaven.test.skip=true
    echo "✓ 镜像构建完成"
}
# 登录Docker仓库
docker_login() {
    if [ "${IS_PROD:-true}" = "true" ]; then
        echo "→ 生产环境模式，执行Docker登录..."
        echo "→ 登录Docker仓库 $DOCKER_REGISTRY..."
        docker login --username="$DOCKER_USERNAME" --password="$DOCKER_PASSWORD" "$DOCKER_REGISTRY"
        echo "✓ 登录成功"
    else
        echo "→ 非生产环境模式，跳过Docker登录"
    fi
}

# 标记和推送镜像
tag_and_push() {
    echo "→ 检查源镜像是否存在: $SOURCE_IMAGE:$IMAGE_VERSION"
    if ! docker image inspect "$SOURCE_IMAGE:$IMAGE_VERSION" &>/dev/null; then
        echo "警告：源镜像不存在，尝试构建..."

        # 这里可以添加构建镜像的逻辑，或者直接报错
        echo "错误：源镜像不存在: $SOURCE_IMAGE:$IMAGE_VERSION"
        echo "可用的镜像列表:"
        docker images | grep "$SOURCE_IMAGE" || echo "没有找到相关镜像"
        exit 1
    fi

    echo "→ 标记镜像: $SOURCE_IMAGE:$IMAGE_VERSION → $TARGET_IMAGE:$IMAGE_VERSION"
    docker tag "$SOURCE_IMAGE:$IMAGE_VERSION" "$TARGET_IMAGE:$IMAGE_VERSION"

    echo "→ 推送镜像到 $TARGET_IMAGE:$IMAGE_VERSION"
    docker push "$TARGET_IMAGE:$IMAGE_VERSION"
    echo "✓ 镜像推送完成"
}

# ========== 主流程 ==========
main() {
    echo "====== 开始执行镜像推送流程 [$(date +'%Y-%m-%d %H:%M:%S')] ======"
    check_required_vars
    make_image
    docker_login
    tag_and_push
    echo "====== 镜像推送流程执行完毕 [耗时: $SECONDS 秒] ======"
}

# 执行主函数
main