#!/bin/bash
set -euo pipefail  # 严格模式：错误退出、未定义变量报错、管道错误检测

# ========== 服务重启脚本，一般用于开发和测试环境直接重启，不使用rancher，jenkins点一下搞定 ==========
# 该脚本负责更新和重启远程服务

# 检查必要的环境变量
check_required_vars() {
    local required_vars=("JAR_NAME" "LOCAL_JAR" "REMOTE_TMP" "CONTAINER_APP_PATH" "IMAGE_PATTERN" "REMOTE_HOST" "RESTART_WAIT_TIME")
    for var in "${required_vars[@]}"; do
        if [ -z "${!var:-}" ]; then
            echo "错误：缺少必要的环境变量 $var"
            exit 1
        fi
    done
    echo "✓ 环境变量检查通过"
}

# 传输JAR包到远程服务器
transfer_jar() {
    echo "→ 开始传输JAR包到远程服务器..."

    # 检查本地JAR包是否存在
    if [ ! -f "$LOCAL_JAR" ]; then
        echo "✗ 本地JAR包不存在: $LOCAL_JAR"
        exit 1
    fi

    scp "$LOCAL_JAR" "$REMOTE_HOST:$REMOTE_TMP" || {
        echo "✗ 文件传输失败"; exit 1
    }
    echo "✓ 文件已传输到远程: $REMOTE_TMP"
}

# 远程执行容器更新
restart_remote_container() {
    echo "→ 连接到远程主机执行容器更新..."

    # 使用不带引号的heredoc，本地变量会在传递前被替换
    ssh "$REMOTE_HOST" <<REMOTE_SCRIPT
    set -e
    echo "→ 正在查找运行中的容器..."
    container_id=\$(docker ps | grep "${IMAGE_PATTERN}" | awk '{print \$1}' | head -1)
    [ -z "\$container_id" ] && { echo "✗ 未找到匹配容器"; exit 1; }
    echo "✓ 找到容器ID: \$container_id"

    echo "→ 更新容器文件..."
    docker cp "${REMOTE_TMP}" "\$container_id:${CONTAINER_APP_PATH}" || {
        echo "✗ 容器内文件复制失败"; exit 1
    }

    echo "→ 重启容器..."
    docker restart "\$container_id"
    sleep ${RESTART_WAIT_TIME}  # 等待重启完成

    echo "→ 验证状态..."
    status=\$(docker inspect -f '{{.State.Status}}' "\$container_id")
    if [ "\$status" = "running" ]; then
        echo "✓ 容器重启成功 (状态: \$status)"
    else
        echo "✗ 容器状态异常: \$status"
        docker logs --tail 20 "\$container_id"
        exit 1
    fi
REMOTE_SCRIPT
}

# ========== 主流程 ==========
main() {
    echo "====== 开始执行服务重启流程 [$(date +'%Y-%m-%d %H:%M:%S')] ======"
    check_required_vars
    transfer_jar
    restart_remote_container
    echo "====== 服务重启流程执行完毕 [耗时: $SECONDS 秒] ======"
}

# 执行主函数
main